@@ -32,7 +32,11 @@ from gnomemusic.query import Query
 from gnomemusic import log, TrackerWrapper
 import logging
 import os
-os.environ['GRL_PLUGIN_RANKS'] = 'local-metadata:3,filesystem:2,tracker:1,lastfm-albumart:0'
+os.environ['GRL_PLUGIN_RANKS'] = ("grl-local-metadata:5,"
+                                  "grl-filesystem:4,"
+                                  "grl-tracker-source:3,"
+                                  "grl-spotify-cover:2,"
+                                  "grl-lastfm-cover:1")
 from gi.repository import Grl
 logger = logging.getLogger(__name__)
 
@@ -46,13 +50,20 @@ class Grilo(GObject.GObject):
     }
 
     METADATA_KEYS = [
-        Grl.METADATA_KEY_ID, Grl.METADATA_KEY_TITLE,
-        Grl.METADATA_KEY_ARTIST, Grl.METADATA_KEY_ALBUM,
-        Grl.METADATA_KEY_DURATION,
+        Grl.METADATA_KEY_ALBUM,
+        Grl.METADATA_KEY_ALBUM_ARTIST,
+        Grl.METADATA_KEY_ALBUM_DISC_NUMBER,
+        Grl.METADATA_KEY_ARTIST,
         Grl.METADATA_KEY_CREATION_DATE,
-        Grl.METADATA_KEY_URL,
+        Grl.METADATA_KEY_COMPOSER,
+        Grl.METADATA_KEY_DURATION,
+        Grl.METADATA_KEY_ID,
         Grl.METADATA_KEY_LYRICS,
-        Grl.METADATA_KEY_THUMBNAIL]
+        Grl.METADATA_KEY_THUMBNAIL,
+        Grl.METADATA_KEY_TITLE,
+        Grl.METADATA_KEY_TRACK_NUMBER,
+        Grl.METADATA_KEY_URL
+    ]
 
     METADATA_THUMBNAIL_KEYS = [
         Grl.METADATA_KEY_ID,
@@ -72,6 +83,8 @@ class Grilo(GObject.GObject):
                                                   "gnome-music", "playlists"])
         if not (GLib.file_test(self.playlist_path, GLib.FileTest.IS_DIR)):
             GLib.mkdir_with_parents(self.playlist_path, int("0755", 8))
+
+        Grl.init(None)
         self.options = Grl.OperationOptions()
         self.options.set_resolution_flags(Grl.ResolutionFlags.FAST_ONLY |
                                           Grl.ResolutionFlags.IDLE_RELAY)
@@ -118,10 +131,11 @@ class Grilo(GObject.GObject):
                     media_id = media.get_id()
                     if changeType == Grl.SourceChangeType.ADDED:
                         # Check that this media is an audio file
-                        query = "select DISTINCT rdf:type nie:mimeType(?urn) as mime-type" +\
-                                " { ?urn rdf:type nie:InformationElement . FILTER (tracker:id(?urn) = %s) }" % media_id
-                        mimeType = grilo.tracker.query_sync(query, [Grl.METADATA_KEY_MIME], grilo.options)[0].get_mime()
-                        if mimeType and mimeType.startswith("audio"):
+                        mime_type = grilo.tracker.query_sync(
+                            Query.is_audio(media_id),
+                            [Grl.METADATA_KEY_MIME],
+                            grilo.options)[0].get_mime()
+                        if mime_type and mime_type.startswith("audio"):
                             self.changed_media_ids.append(media_id)
                     if changeType == Grl.SourceChangeType.REMOVED:
                         # There is no way to check that removed item is a media
@@ -170,6 +184,15 @@ class Grilo(GObject.GObject):
 
     @log
     def _on_source_added(self, pluginRegistry, mediaSource):
+        if ("net:plaintext" in mediaSource.get_tags()
+                or mediaSource.get_id() in self.blacklist):
+            try:
+                pluginRegistry.unregister_source(mediaSource)
+            except GLib.GError:
+                logger.error("Failed to unregister %s.",
+                             mediaSource.get_id())
+            return
+
         id = mediaSource.get_id()
         logger.debug("new grilo source %s was added", id)
         try:
@@ -194,8 +217,8 @@ class Grilo(GObject.GObject):
                 self.sources[id] = mediaSource
                 self.emit('new-source-added', mediaSource)
 
-            elif (id not in self.blacklist) and (ops & Grl.SupportedOps.SEARCH) and \
-                 (mediaSource.get_supported_media() & Grl.MediaType.AUDIO):
+            elif (ops & Grl.SupportedOps.SEARCH
+                  and mediaSource.get_supported_media() & Grl.MediaType.AUDIO):
                 logger.debug("source %s is searchable", id)
                 self.sources[id] = mediaSource
                 self.emit('new-source-added', mediaSource)
@@ -264,6 +287,16 @@ class Grilo(GObject.GObject):
             self.sparqltracker.update(Query.add_favorite(song_item.get_url()), GLib.PRIORITY_DEFAULT, None)
             song_item.set_lyrics("i'm truthy")
 
+    @log
+    def set_favorite(self, song_item, favorite):
+        """Set the favorite status of a media item
+
+        :param song_item: A Grilo media item
+        :param bool favorite: Set favorite status
+        """
+        if bool(song_item.get_lyrics()) != favorite:
+            self.toggle_favorite(song_item)
+
     @log
     def search(self, q, callback, data=None):
         options = self.options.copy()
@@ -290,10 +323,9 @@ class Grilo(GObject.GObject):
                                 _multiple_search_callback, data)
 
     @log
-    def get_album_art_for_item(self, item, callback, data=None):
+    def get_album_art_for_item(self, item, callback):
         item_id = item.get_id()
 
-        query = None
         if item.is_audio():
             query = Query.get_album_for_song_id(item_id)
         else:
@@ -302,7 +334,8 @@ class Grilo(GObject.GObject):
         options = self.full_options.copy()
         options.set_count(1)
 
-        self.search_source.query(query, self.METADATA_THUMBNAIL_KEYS, options, callback, data)
+        self.search_source.query(query, self.METADATA_THUMBNAIL_KEYS, options,
+                                 callback)
 
     @log
     def get_playlist_with_id(self, playlist_id, callback):
@@ -319,6 +352,4 @@ class Grilo(GObject.GObject):
         self.tracker.query(query, self.METADATA_KEYS, options, callback, None)
 
 
-Grl.init(None)
-
 grilo = Grilo()
