@@ -42,7 +42,7 @@ from gi.repository import Gtk, Gdk, GLib, Gio, GObject, Gst, GstAudio, GstPbutil
 from gettext import gettext as _, ngettext
 from random import randint
 from collections import deque
-from gnomemusic.albumartcache import AlbumArtCache, DefaultIcon
+from gnomemusic.albumartcache import AlbumArtCache, DefaultIcon, ArtSize
 from gnomemusic.playlists import Playlists
 import gnomemusic.utils as utils
 playlists = Playlists.get_default()
@@ -56,8 +56,6 @@ from gnomemusic import log
 import logging
 logger = logging.getLogger(__name__)
 
-ART_SIZE = 34
-
 
 class RepeatType:
     NONE = 0
@@ -93,7 +91,7 @@ class Player(GObject.GObject):
         'volume-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'prev-next-invalidated': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'seeked': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
-        'thumbnail-updated': (GObject.SignalFlags.RUN_FIRST, None, (str,)),
+        'thumbnail-updated': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
     def __repr__(self):
@@ -110,10 +108,11 @@ class Player(GObject.GObject):
         self.currentTrack = None
         self.currentTrackUri = None
         self._lastState = Gst.State.PAUSED
-        self.cache = AlbumArtCache.get_default()
-        self._no_artwork_icon = DefaultIcon().get(ART_SIZE,
-                                                  ART_SIZE,
-                                                  DefaultIcon.Type.music)
+        scale = parent_window.get_scale_factor()
+        self.cache = AlbumArtCache(scale)
+        self._loading_icon_surface = DefaultIcon(scale).get(
+            DefaultIcon.Type.loading,
+            ArtSize.xsmall)
         self._missingPluginMessages = []
 
         Gst.init(None)
@@ -608,11 +607,10 @@ class Player(GObject.GObject):
         except:
             self._currentAlbum = album
 
-        self.coverImg.set_from_pixbuf(self._no_artwork_icon)
-        self.cache.lookup(
-            media, ART_SIZE, ART_SIZE, self._on_cache_lookup, None, artist, album)
+        self.coverImg.set_from_surface(self._loading_icon_surface)
+        self.cache.lookup(media, ArtSize.xsmall, self._on_cache_lookup, None)
 
-        self._currentTitle = AlbumArtCache.get_media_title(media)
+        self._currentTitle = utils.get_media_title(media)
         self.titleLabel.set_label(self._currentTitle)
 
         self._currentTimestamp = int(time.time())
@@ -663,10 +661,9 @@ class Player(GObject.GObject):
         return False
 
     @log
-    def _on_cache_lookup(self, pixbuf, path, data=None):
-        if pixbuf is not None:
-            self.coverImg.set_from_pixbuf(pixbuf)
-        self.emit('thumbnail-updated', path)
+    def _on_cache_lookup(self, surface, data=None):
+        self.coverImg.set_from_surface(surface)
+        self.emit('thumbnail-updated')
 
     @log
     def play(self):
@@ -810,6 +807,9 @@ class Player(GObject.GObject):
         self.titleLabel = self._ui.get_object('title')
         self.artistLabel = self._ui.get_object('artist')
         self.coverImg = self._ui.get_object('cover')
+        self.coverImg.set_property("width-request", ArtSize.xsmall.width)
+        self.coverImg.set_property("height-request", ArtSize.xsmall.height)
+
         self.duration = self._ui.get_object('duration')
         self.repeatBtnImage = self._ui.get_object('playlistRepeat')
 
