@@ -40,37 +40,40 @@ import logging
 logger = logging.getLogger(__name__)
 
 
+class Playlist:
+    """ Base class of static and intelligent playlists """
+    ID = None
+    QUERY = None
+    TAG_TEXT = ""
+    TITLE = ""
+
+
 class StaticPlaylists:
 
     def __repr__(self):
         return '<StaticPlaylists>'
 
-    class MostPlayed:
-        ID = None
+    class MostPlayed(Playlist):
         TAG_TEXT = "MOST_PLAYED"
         # TRANSLATORS: this is a playlist name
         TITLE = _("Most Played")
 
-    class NeverPlayed:
-        ID = None
+    class NeverPlayed(Playlist):
         TAG_TEXT = "NEVER_PLAYED"
         # TRANSLATORS: this is a playlist name
         TITLE = _("Never Played")
 
-    class RecentlyPlayed:
-        ID = None
+    class RecentlyPlayed(Playlist):
         TAG_TEXT = "RECENTLY_PLAYED"
         # TRANSLATORS: this is a playlist name
         TITLE = _("Recently Played")
 
-    class RecentlyAdded:
-        ID = None
+    class RecentlyAdded(Playlist):
         TAG_TEXT = "RECENTLY_ADDED"
         # TRANSLATORS: this is a playlist name
         TITLE = _("Recently Added")
 
-    class Favorites:
-        ID = None
+    class Favorites(Playlist):
         TAG_TEXT = "FAVORITES"
         # TRANSLATORS: this is a playlist name
         TITLE = _("Favorite Songs")
@@ -84,9 +87,23 @@ class StaticPlaylists:
         self.Favorites.QUERY = Query.get_favorite_songs()
 
     @staticmethod
-    def get_protected_ids():
-        return [str(cls.ID) for name, cls in inspect.getmembers(StaticPlaylists)
-                if inspect.isclass(cls) and not (name == "__class__")]
+    def get_ids():
+        """Get all static playlist IDs
+
+        :return: A list of tracker.id's
+        :rtype: A list of integers
+        """
+        return [str(playlist.ID) for playlist in StaticPlaylists.get_all()]
+
+    @staticmethod
+    def get_all():
+        """Get all static playlist classes
+
+        :return: All StaticPlaylists innerclasses
+        :rtype: A list of classes
+        """
+        return [cls for name, cls in inspect.getmembers(StaticPlaylists)
+                if inspect.isclass(cls) and not name == "__class__"]
 
 
 class Playlists(GObject.GObject):
@@ -119,34 +136,104 @@ class Playlists(GObject.GObject):
     def __init__(self):
         GObject.GObject.__init__(self)
         self.tracker = TrackerWrapper().tracker
-        StaticPlaylists()
+        self._static_playlists = StaticPlaylists()
+
+        grilo.connect('ready', self._on_grilo_ready)
 
     @log
-    def fetch_or_create_static_playlists(self):
+    def _on_grilo_ready(self, data=None):
         """For all static playlists: get ID, if exists; if not, create the playlist and get ID."""
-        playlists = [cls for name, cls in inspect.getmembers(StaticPlaylists)
-                     if inspect.isclass(cls) and not (name == "__class__")]  # hacky
 
-        def callback(obj, result, playlist):
-            cursor = obj.query_finish(result)
-            while (cursor.next(None)):
-                playlist.ID = cursor.get_integer(1)
+        def playlist_id_fetched_cb(cursor, res, playlist):
+            """ Called after the playlist id is fetched """
+            try:
+                cursor.next_finish(res)
+            except GLib.Error as error:
+                logger.warn("Error: %s, %s", err.__class__, err)
+                return
+
+            playlist.ID = cursor.get_integer(1)
 
             if not playlist.ID:
-                # create the playlist
-                playlist.ID = self.create_playlist_and_return_id(playlist.TITLE, playlist.TAG_TEXT)
+                # Create the  static playlist
+                self._create_static_playlist(playlist)
+            else:
+                # Update playlist
+                self.update_static_playlist(playlist)
 
-            self.update_static_playlist(playlist)
+        def callback(obj, result, playlist):
+            """ Starts retrieving the playlist id """
+            try:
+                cursor = obj.query_finish(result)
+            except GLib.Error as error:
+                logger.warn("Error: %s, %s", err.__class__, err)
+                return
 
-        for playlist in playlists:
+            # Search for the playlist ID
+            cursor.next_async(None, playlist_id_fetched_cb, playlist)
+
+        # Start fetching all the static playlists
+        for playlist in self._static_playlists.get_all():
             self.tracker.query_async(
                 Query.get_playlist_with_tag(playlist.TAG_TEXT), None,
                 callback, playlist)
 
     @log
-    def clear_playlist_with_id(self, playlist_id):
-        query = Query.clear_playlist_with_id(playlist_id)
-        self.tracker.update(query, GLib.PRIORITY_LOW, None)
+    def _create_static_playlist(self, playlist):
+        """ Create the tag and the static playlist, and fetch the newly created
+        playlist's songs.
+        """
+        title = playlist.TITLE
+        tag_text = playlist.TAG_TEXT
+
+        def playlist_next_async_cb(cursor, res, playlist):
+            """ Called after we finished moving the Tracker cursor, and ready
+            to retrieve the playlist id"""
+            # Update the playlist ID
+            try:
+                cursor.next_finish(res)
+            except GLib.Error as error:
+                logger.warn("Error: %s, %s", err.__class__, err)
+                return
+
+            playlist.ID = cursor.get_integer(0)
+
+            # Fetch the playlist contents
+            self.update_static_playlist(playlist)
+
+        def playlist_queried_cb(obj, res, playlist):
+            """ Called after the playlist is created and the ID is fetched """
+            try:
+                cursor = obj.query_finish(res)
+            except GLib.Error as error:
+                logger.warn("Error: %s, %s", err.__class__, err)
+                return
+
+            cursor.next_async(None, playlist_next_async_cb, playlist)
+
+        def playlist_created_cb(obj, res, playlist):
+            """ Called when the static playlist is created """
+            data = obj.update_blank_finish(res)
+            playlist_urn = data.get_child_value(0).get_child_value(0).\
+                           get_child_value(0).get_child_value(1).get_string()
+
+            query = Query.get_playlist_with_urn(playlist_urn)
+
+            # Start fetching the playlist
+            self.tracker.query_async(query, None, playlist_queried_cb, playlist)
+
+        def tag_created_cb(obj, res, playlist):
+            """ Called when the tag is created """
+            creation_query = Query.create_playlist_with_tag(title, tag_text)
+
+            # Start creating the playlist itself
+            self.tracker.update_blank_async(creation_query, GLib.PRIORITY_LOW,
+                                            None, playlist_created_cb, playlist)
+
+        # Start the playlist creation by creating the tag
+        self.tracker.update_blank_async(Query.create_tag(tag_text),
+                                        GLib.PRIORITY_LOW, None,
+                                        tag_created_cb, playlist)
 
     @log
     def update_playcount(self, song_url):
@@ -163,51 +250,63 @@ class Playlists(GObject.GObject):
     def update_static_playlist(self, playlist):
         """Given a static playlist (subclass of StaticPlaylists), updates according to its query."""
         # Clear the playlist
-        self.clear_playlist_with_id(playlist.ID)
+        self.clear_playlist(playlist)
 
+    @log
+    def clear_playlist(self, playlist):
+        """Starts cleaning the playlist"""
+        query = Query.clear_playlist_with_id(playlist.ID)
+        self.tracker.update_async(query, GLib.PRIORITY_LOW, None,
+                                  self._static_playlist_cleared_cb, playlist)
+
+    @log
+    def _static_playlist_cleared_cb(self, connection, res, playlist):
+        """After clearing the playlist, start querying the playlist's songs"""
+        # Get a list of matching songs
+        self.tracker.query_async(playlist.QUERY, None,
+                                 self._static_playlist_query_cb, playlist)
+
+    @log
+    def _static_playlist_query_cb(self, connection, res, playlist):
+        """Fetch the playlist's songs"""
         final_query = ''
 
         # Get a list of matching songs
-        cursor = self.tracker.query(playlist.QUERY, None)
-        if not cursor:
+        try:
+            cursor = self.tracker.query_finish(res)
+        except GLib.Error as err:
+            logger.warn("Error: %s, %s", err.__class__, err)
             return
 
-        # For each song run 'add song to playlist'
-        while cursor.next():
+        def callback(cursor, res, final_query):
             uri = cursor.get_string(0)[0]
             final_query += Query.add_song_to_playlist(playlist.ID, uri)
 
-        self.tracker.update_blank_async(final_query, GLib.PRIORITY_LOW,
-                                        None, None, None)
+            try:
+                has_next = cursor.next_finish(res)
+            except GLib.Error as err:
+                logger.warn("Error: %s, %s", err.__class__, err)
+                has_next = False
 
-        # tell system we updated the playlist so playlist is reloaded
-        self.emit('playlist-updated', playlist.ID)
+            # Only perform the update when the cursor reached the end
+            if has_next:
+                cursor.next_async(None, callback, final_query)
+                return
+
+            self.tracker.update_blank_async(final_query, GLib.PRIORITY_LOW,
+                                            None, None, None)
+
+            # tell system we updated the playlist so playlist is reloaded
+            self.emit('playlist-updated', playlist.ID)
+
+        # Asynchronously form the playlist's final query
+        cursor.next_async(None, callback, final_query)
 
     @log
     def update_all_static_playlists(self):
-        playlists = [cls for name, cls in inspect.getmembers(StaticPlaylists)
-                     if inspect.isclass(cls) and not (name == "__class__")]  # hacky
-
-        for playlist in playlists:
+        for playlist in self._static_playlists.get_all():
             self.update_static_playlist(playlist)
 
-    @log
-    def create_playlist_and_return_id(self, title, tag_text):
-        self.tracker.update_blank(Query.create_tag(tag_text), GLib.PRIORITY_LOW, None)
-
-        data = self.tracker.update_blank(
-            Query.create_playlist_with_tag(title, tag_text), GLib.PRIORITY_LOW,
-            None)
-        playlist_urn = data.get_child_value(0).get_child_value(0).\
-            get_child_value(0).get_child_value(1).get_string()
-
-        cursor = self.tracker.query(
-            Query.get_playlist_with_urn(playlist_urn),
-            None)
-        if not cursor or not cursor.next():
-            return
-        return cursor.get_integer(0)
-
     @log
     def create_playlist(self, title):
         def get_callback(source, param, item, count, data, error):
@@ -293,9 +392,12 @@ class Playlists(GObject.GObject):
 
     @log
     def is_static_playlist(self, playlist):
-        """Checks whether the given playlist is static or not"""
+        """Checks whether the given playlist is static or not
 
-        for static_playlist_id in StaticPlaylists.get_protected_ids():
+        :return: True if the playlist is static
+        :rtype: bool
+        """
+        for static_playlist_id in self._static_playlists.get_ids():
             if playlist.get_id() == static_playlist_id:
                 return True
 
