@@ -84,13 +84,13 @@ class Query():
         }
 
         # TRANSLATORS: The following translatable string should be a
-        # space-separated list of all-lowercase articles that should be
-        # ignored when alphabetizing artists/albums. This list should
-        # include the basic english translatable strings regardless of
-        # language because it is so universal.
+        # vertical bar-separated list of all-lowercase articles that
+        # should be ignored when alphabetizing artists/albums. This
+        # list should include the basic english translatable strings
+        # regardless of language because they are so universal.
         # If some articles occur more frequently than others, the most
         # common one should appear first, the least common one last.
-        for article in reversed(_("the a an").split(" ")):
+        for article in reversed(_("the|a|an").split("|")):
             return_statement = """IF(STRSTARTS(%(attribute)s, "%(article)s"),
             SUBSTR(%(attribute)s, %(substr_start)s), %(nested_if)s)
             """ % {
@@ -119,7 +119,7 @@ class Query():
 
     @staticmethod
     def all_songs_count():
-        query = '''
+        query = """
     SELECT
         COUNT(?song) AS ?childcount
     {
@@ -128,7 +128,7 @@ class Query():
 	      nie:url ?url .
         FILTER(STRSTARTS(?url, '%(music_dir)s/'))
     }
-    '''.replace('\n', ' ').strip() % {
+    """.replace('\n', ' ').strip() % {
             'music_dir': Query.MUSIC_URI
         }
 
@@ -136,11 +136,12 @@ class Query():
 
     @staticmethod
     def albums(where_clause):
-        query = '''
+        query = """
     SELECT
         rdf:type(?album)
         tracker:id(?album) AS ?id
         nmm:artistName(?albumArtist) AS ?album_artist
+        nmm:artistName(?composer) AS ?composer
         nmm:artistName(?performer) AS ?artist
         ?title
         COUNT(?song) AS ?childcount
@@ -156,11 +157,12 @@ class Query():
         BIND(LCASE(nmm:artistName(?albumArtist)) AS ?artist_lower) .
         BIND((%(album_order)s) AS ?album_collation) .
         BIND((%(artist_order)s) AS ?artist_collation) .
+        OPTIONAL { ?song nmm:composer ?composer . }
         FILTER(STRSTARTS(nie:url(?song), '%(music_dir)s/'))
     }
     GROUP BY ?album
     ORDER BY ?album_collation ?artist_collation ?creation_date
-    '''.replace('\n', ' ').strip() % {
+    """.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
             'music_dir': Query.MUSIC_URI,
             'album_order': Query._order_by_statement("?title_lower"),
@@ -171,7 +173,7 @@ class Query():
 
     @staticmethod
     def artists(where_clause):
-        query = '''
+        query = """
     SELECT
         rdf:type(?album)
         tracker:id(?album) AS ?id
@@ -195,7 +197,7 @@ class Query():
     }
     GROUP BY ?album
     ORDER BY ?artist_collation ?creation_date ?album_collation
-    '''.replace('\n', ' ').strip() % {
+    """.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
             'music_dir': Query.MUSIC_URI,
             'artist_order': Query._order_by_statement("?artist_lower"),
@@ -206,7 +208,7 @@ class Query():
 
     @staticmethod
     def songs(where_clause):
-        query = '''
+        query = """
     SELECT DISTINCT
         rdf:type(?song)
         tracker:id (?song) AS ?id
@@ -230,7 +232,7 @@ class Query():
         FILTER(STRSTARTS(?url, '%(music_dir)s/'))
     }
     ORDER BY ?artist ?album nmm:setNumber(?disc) nmm:trackNumber(?song)
-    '''.replace('\n', ' ').strip() % {
+    """.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
             'music_dir': Query.MUSIC_URI
         }
@@ -239,7 +241,7 @@ class Query():
 
     @staticmethod
     def playlists(where_clause):
-        query = '''
+        query = """
     SELECT DISTINCT
         rdf:type(?playlist)
         tracker:id(?playlist) AS ?id
@@ -254,7 +256,7 @@ class Query():
             OPTIONAL { ?playlist nao:hasTag ?tag }
         }
     ORDER BY !BOUND(?tag) LCASE(?title)
-    '''.replace('\n', ' ').strip() % {
+    """.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
             'music_dir': Query.MUSIC_URI
         }
@@ -263,7 +265,7 @@ class Query():
 
     @staticmethod
     def album_songs(album_id):
-        query = '''
+        query = """
     SELECT DISTINCT
         rdf:type(?song)
         tracker:id(?song) AS ?id
@@ -272,6 +274,8 @@ class Query():
         nmm:artistName(nmm:performer(?song)) AS ?artist
         nie:title(nmm:musicAlbum(?song)) AS ?album
         nfo:duration(?song) AS ?duration
+        nmm:trackNumber(?song) AS ?track_number
+        nmm:setNumber(nmm:musicAlbumDisc(?song)) AS ?album_disc_number
         IF(bound(?tag), 'truth!', '') AS ?lyrics
     WHERE {
         ?song a nmm:MusicPiece ;
@@ -283,10 +287,10 @@ class Query():
                 (STRSTARTS(nie:url(?song), '%(music_dir)s/')))
     }
     ORDER BY
-         nmm:setNumber(nmm:musicAlbumDisc(?song))
-         nmm:trackNumber(?song)
+         ?album_disc_number
+         ?track_number
          tracker:added(?song)
-    '''.replace('\n', ' ').strip() % {
+    """.replace('\n', ' ').strip() % {
             'album_id': album_id,
             'music_dir': Query.MUSIC_URI
         }
@@ -295,7 +299,7 @@ class Query():
 
     @staticmethod
     def playlist_songs(playlist_id, filter_clause=None):
-        query = '''
+        query = """
     SELECT
         rdf:type(?song)
         tracker:id(?entry) AS ?id
@@ -332,7 +336,7 @@ class Query():
     }
     ORDER BY
          nfo:listPosition(?entry)
-    '''.replace('\n', ' ').strip() % {
+    """.replace('\n', ' ').strip() % {
             'playlist_id': playlist_id,
             'filter_clause': filter_clause or 'tracker:id(?playlist) = ' + playlist_id,
             'music_dir': Query.MUSIC_URI
@@ -393,9 +397,7 @@ class Query():
             tracker:id(?song) = %(song_id)s
         )
         FILTER (
-            tracker:uri-is-descendant(
-                '%(music_dir)s', nie:url(?song)
-            )
+            STRSTARTS(nie:url(?song), '%(music_dir)s')
         )
         FILTER (
             NOT EXISTS {
@@ -723,11 +725,7 @@ class Query():
                 nie:usageCounter ?count ;
                 nie:isStoredAs ?as .
           ?as nie:url ?url .
-          FILTER (
-            tracker:uri-is-descendant(
-              '%(music_dir)s', ?url
-            )
-          )
+          FILTER ( STRSTARTS(?url, '%(music_dir)s') )
         } ORDER BY DESC(?count) LIMIT 50
         """.replace('\n', ' ').strip() % {
             'music_dir': Query.MUSIC_URI
@@ -743,12 +741,8 @@ class Query():
             ?song a nmm:MusicPiece ;
                 nie:isStoredAs ?as .
             ?as nie:url ?url .
-            FILTER ( NOT EXISTS { ?song nie:usageCounter ?count .} )
-            FILTER (
-            tracker:uri-is-descendant(
-                '%(music_dir)s', ?url
-            )
-        )
+            FILTER ( NOT EXISTS { ?song nie:usageCounter ?count .}
+                     && STRSTARTS(?url, '%(music_dir)s') )
         } ORDER BY nfo:fileLastAccessed(?song) LIMIT 50
         """.replace('\n', ' ').strip() % {
             'music_dir': Query.MUSIC_URI
@@ -773,13 +767,9 @@ class Query():
                     nie:isStoredAs ?as ;
                     nfo:fileLastAccessed ?last_played .
                 ?as nie:url ?url .
-                FILTER ( ?last_played > '%(compare_date)s'^^xsd:dateTime )
-                FILTER ( EXISTS { ?song nie:usageCounter ?count .} )
-                FILTER (
-                  tracker:uri-is-descendant(
-                    '%(music_dir)s', ?url
-                  )
-                )
+                FILTER ( ?last_played > '%(compare_date)s'^^xsd:dateTime
+                         && EXISTS { ?song nie:usageCounter ?count .}
+                         && STRSTARTS(?url, '%(music_dir)s') )
             } ORDER BY DESC(?last_played) LIMIT 50
             """.replace('\n', ' ').strip() % {
                 'compare_date': compare_date,
@@ -806,12 +796,8 @@ class Query():
                 nie:isStoredAs ?as ;
                 tracker:added ?added .
             ?as nie:url ?url .
-            FILTER ( ?added > '%(compare_date)s'^^xsd:dateTime )
-            FILTER (
-              tracker:uri-is-descendant(
-                '%(music_dir)s', ?url
-              )
-            )
+            FILTER ( ?added > '%(compare_date)s'^^xsd:dateTime
+                     && STRSTARTS(?url, '%(music_dir)s') )
         } ORDER BY DESC(?added) LIMIT 50
         """.replace('\n', ' ').strip() % {
             'compare_date': compare_date,
@@ -829,11 +815,7 @@ class Query():
             nie:isStoredAs ?as ;
             nao:hasTag nao:predefined-tag-favorite .
         ?as nie:url ?url .
-        FILTER (
-          tracker:uri-is-descendant(
-            '%(music_dir)s', nie:url(?as)
-          )
-        )
+        FILTER ( STRSTARTS(?url, '%(music_dir)s') )
     } ORDER BY DESC(tracker:added(?song))
     """.replace('\n', ' ').strip() % {
             'music_dir': Query.MUSIC_URI,
@@ -846,7 +828,7 @@ class Query():
     @staticmethod
     def get_albums_with_any_match(name):
         name = Tracker.sparql_escape_string(GLib.utf8_normalize(GLib.utf8_casefold(name, -1), -1, GLib.NormalizeMode.NFKD))
-        query = '''
+        query = """
             {
                 SELECT DISTINCT
                     nmm:musicAlbum(?song) AS ?album
@@ -865,35 +847,47 @@ class Query():
                     )
                 }
             }
-            '''.replace('\n', ' ').strip() % {'name': name}
+            """.replace('\n', ' ').strip() % {'name': name}
 
         return Query.albums(query)
 
     @staticmethod
     def get_albums_with_artist_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = '?performer fts:match "nmm:artistName: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
+        query = """?performer fts:match '"nmm:artistName" : %(name)s*' . """.replace('\n', ' ').strip() % {'name': name}
 
         return Query.albums(query)
 
     @staticmethod
     def get_albums_with_album_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = '?album fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
+        query = """?album fts:match '"nie:title" : %(name)s*' . """.replace('\n', ' ').strip() % {'name': name}
+
+        return Query.albums(query)
+
+    @staticmethod
+    def get_albums_with_composer_match(name):
+        name = Tracker.sparql_escape_string(name)
+        query = """
+            ?song nmm:composer ?composer .
+            ?composer fts:match '"nmm:artistName" : %(name)s*' .
+        """.replace('\n', ' ').strip() % {
+            'name': name
+        }
 
         return Query.albums(query)
 
     @staticmethod
     def get_albums_with_track_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = '?song fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
+        query = """?song fts:match '"nie:title" : %(name)s*' . """.replace('\n', ' ').strip() % {'name': name}
 
         return Query.albums(query)
 
     @staticmethod
     def get_artists_with_any_match(name):
         name = Tracker.sparql_escape_string(GLib.utf8_normalize(GLib.utf8_casefold(name, -1), -1, GLib.NormalizeMode.NFKD))
-        query = '''
+        query = """
             {
                 SELECT DISTINCT
                     nmm:musicAlbum(?song) AS ?album
@@ -912,35 +906,47 @@ class Query():
                     )
                 }
             }
-            '''.replace('\n', ' ').strip() % {'name': name}
+            """.replace('\n', ' ').strip() % {'name': name}
 
         return Query.artists(query)
 
     @staticmethod
     def get_artists_with_artist_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = '?performer fts:match "nmm:artistName: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
+        query = """?performer fts:match '"nmm:artistName" : %(name)s*' . """.replace('\n', ' ').strip() % {'name': name}
 
         return Query.artists(query)
 
     @staticmethod
     def get_artists_with_album_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = '?album fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
+        query = """?album fts:match '"nie:title" : %(name)s*' . """.replace('\n', ' ').strip() % {'name': name}
+
+        return Query.artists(query)
+
+    @staticmethod
+    def get_artists_with_composer_match(name):
+        name = Tracker.sparql_escape_string(name)
+        query = """
+            ?song nmm:composer ?composer .
+            ?composer fts:match '"nmm:artistName" : %(name)s*' .
+        """.replace('\n', ' ').strip() % {
+            'name': name
+        }
 
         return Query.artists(query)
 
     @staticmethod
     def get_artists_with_track_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = '?song fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
+        query = """?song fts:match '"nie:title" : %(name)s*' . """.replace('\n', ' ').strip() % {'name': name}
 
         return Query.artists(query)
 
     @staticmethod
     def get_songs_with_any_match(name):
         name = Tracker.sparql_escape_string(GLib.utf8_normalize(GLib.utf8_casefold(name, -1), -1, GLib.NormalizeMode.NFKD))
-        query = '''
+        query = """
             {
                 SELECT DISTINCT
                     ?song
@@ -959,28 +965,40 @@ class Query():
                     )
                 }
             }
-            '''.replace('\n', ' ').strip() % {'name': name}
+            """.replace('\n', ' ').strip() % {'name': name}
 
         return Query.songs(query)
 
     @staticmethod
     def get_songs_with_artist_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = '?performer fts:match "nmm:artistName: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
+        query = """?performer fts:match '"nmm:artistName" : %(name)s*' . """.replace('\n', ' ').strip() % {'name': name}
 
         return Query.songs(query)
 
     @staticmethod
     def get_songs_with_album_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = '?album fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
+        query = """?album fts:match '"nie:title" : %(name)s*' . """.replace('\n', ' ').strip() % {'name': name}
+
+        return Query.songs(query)
+
+    @staticmethod
+    def get_songs_with_composer_match(name):
+        name = Tracker.sparql_escape_string(name)
+        query = """
+            ?song nmm:composer ?composer .
+            ?composer fts:match '"nmm:artistName" : %(name)s*' .
+        """.replace('\n', ' ').strip() % {
+            'name': name
+        }
 
         return Query.songs(query)
 
     @staticmethod
     def get_songs_with_track_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = '?song fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
+        query = """?song fts:match '"nie:title" : %(name)s*' . """.replace('\n', ' ').strip() % {'name': name}
 
         return Query.songs(query)
 
@@ -1053,3 +1071,19 @@ class Query():
         }
 
         return query
+
+    @staticmethod
+    def is_audio(media_id):
+        query = """
+            SELECT DISTINCT
+            rdf:type
+            nie:mimeType(?urn) AS mime_type
+            {
+                ?urn rdf:type nie:InformationElement .
+                FILTER ( tracker:id(?urn) = "%(media_id)s" )
+            }
+        """.replace('\n', '').strip() % {
+            'media_id' : media_id
+        }
+
+        return query
