@@ -25,6 +25,25 @@
 from gettext import gettext as _
 
 
+def get_album_title(item):
+    """Returns the album title associated with the media item
+
+    In case of an audio file the get_album call returns the
+    album title and in case of a container we are looking for
+    the title.
+
+    :param item: A Grilo Media object
+    :return: The album title
+    :rtype: string
+    """
+    album = item.get_album()
+
+    if not album:
+        album = get_media_title(item)
+
+    return album
+
+
 def get_artist_name(item):
     """Returns the preferred artist for a media item.
 
@@ -42,3 +61,29 @@ def get_artist_name(item):
     return (item.get_album_artist()
             or item.get_artist()
             or _("Unknown Artist"))
+
+
+def get_media_title(item):
+    """Returns the title of the media item.
+
+    :param item: A Grilo Media object
+    :return: The title
+    :rtype:
+    """
+
+    return (item.get_title()
+            or _("Untitled"))
+
+
+def seconds_to_string(duration):
+    """Convert a time in seconds to a hh:mm string
+
+    :param int duration: Time in seconds
+    :return: Time in hh:mm format
+    :rtype: string
+    """
+    seconds = duration
+    minutes = seconds // 60
+    seconds %= 60
+
+    return '{:d}:{:02d}'.format(minutes, seconds)
