@@ -34,15 +34,23 @@ gi.require_version('Gd', '1.0')
 from gi.repository import Gtk, Gdk, Gio, GLib, Gd
 from gettext import gettext as _, ngettext
 
+from gnomemusic import log
 from gnomemusic import TrackerWrapper
 from gnomemusic.toolbar import Toolbar, ToolbarState
 from gnomemusic.player import Player, SelectionToolbar, RepeatType
 from gnomemusic.query import Query
-import gnomemusic.view as Views
-import gnomemusic.widgets as Widgets
+from gnomemusic.views.albumsview import AlbumsView
+from gnomemusic.views.artistsview import ArtistsView
+from gnomemusic.views.emptyview import EmptyView
+from gnomemusic.views.emptysearchview import EmptySearchView
+from gnomemusic.views.initialstateview import InitialStateView
+from gnomemusic.views.searchview import SearchView
+from gnomemusic.views.songsview import SongsView
+from gnomemusic.views.playlistview import PlaylistView
+from gnomemusic.widgets.playlistdialog import PlaylistDialog
 from gnomemusic.playlists import Playlists
 from gnomemusic.grilo import grilo
-from gnomemusic import log
+
 import logging
 logger = logging.getLogger(__name__)
 
@@ -248,9 +256,9 @@ class Window(Gtk.ApplicationWindow):
         did_initial_state = self.settings.get_boolean('did-initial-state')
         view_class = None
         if did_initial_state:
-            view_class = Views.Empty
+            view_class = EmptyView
         else:
-            view_class = Views.InitialState
+            view_class = InitialStateView
         self.views.append(view_class(self, self.player))
 
         self._stack.add_titled(self.views[0], _("Empty"), _("Empty"))
@@ -264,12 +272,12 @@ class Window(Gtk.ApplicationWindow):
         self.connect('destroy', self._notify_mode_disconnect)
         self._key_press_event_id = self.connect('key_press_event', self._on_key_press)
 
-        self.views.append(Views.Albums(self, self.player))
-        self.views.append(Views.Artists(self, self.player))
-        self.views.append(Views.Songs(self, self.player))
-        self.views.append(Views.Playlist(self, self.player))
-        self.views.append(Views.Search(self, self.player))
-        self.views.append(Views.EmptySearch(self, self.player))
+        self.views.append(AlbumsView(self, self.player))
+        self.views.append(ArtistsView(self, self.player))
+        self.views.append(SongsView(self, self.player))
+        self.views.append(PlaylistView(self, self.player))
+        self.views.append(SearchView(self, self.player))
+        self.views.append(EmptySearchView(self, self.player))
 
         for i in self.views:
             if i.title:
@@ -298,10 +306,10 @@ class Window(Gtk.ApplicationWindow):
     def _on_select_none(self, action, param):
         if self.toolbar._state == ToolbarState.MAIN:
             view = self._stack.get_visible_child()
+            view.unselect_all()
         else:
             view = self._stack.get_visible_child().get_visible_child()
-
-        view.unselect_all()
+            view.select_none()
 
     def _show_notification(self):
         self.notification_handler = None
@@ -331,7 +339,7 @@ class Window(Gtk.ApplicationWindow):
         if self.pl_todelete_notification:
             self.views[3].really_delete = False
             self.pl_todelete_notification.destroy()
-            Views.playlists.delete_playlist(self.views[3].pl_todelete)
+            playlist.delete_playlist(self.views[3].pl_todelete)
 
         self.notification = Gd.Notification()
         self.notification.set_timeout(20)
@@ -357,7 +365,7 @@ class Window(Gtk.ApplicationWindow):
     def _playlist_removal_notification_dismissed(self, widget):
         self.pl_todelete_notification = None
         if self.views[3].really_delete:
-            Views.playlists.delete_playlist(self.views[3].pl_todelete)
+            playlist.delete_playlist(self.views[3].pl_todelete)
         else:
             self.views[3].really_delete = True
 
@@ -512,7 +520,7 @@ class Window(Gtk.ApplicationWindow):
             if len(selected_tracks) < 1:
                 return
 
-            add_to_playlist = Widgets.PlaylistDialog(self)
+            add_to_playlist = PlaylistDialog(self)
             if add_to_playlist.dialog_box.run() == Gtk.ResponseType.ACCEPT:
                 playlist.add_to_playlist(
                     add_to_playlist.get_selected(),
